@@ -5,7 +5,6 @@
 #![deny(rustdoc::broken_intra_doc_links)]
 #![deny(rustdoc::invalid_codeblock_attributes)]
 
-use std::convert::TryFrom;
 use std::fmt;
 
 use http::HeaderValue;
@@ -213,6 +212,41 @@ pub enum BodyWriteEnd {
     Front = 1,
 }
 
+/// Determines how the framing headers (`Content-Length`/`Transfer-Encoding`) are set for a
+/// request or response.
+#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[repr(u32)]
+pub enum FramingHeadersMode {
+    /// Determine the framing headers automatically based on the message body, and discard any framing
+    /// headers already set in the message. This is the default behavior.
+    ///
+    /// In automatic mode, a `Content-Length` is used when the size of the body can be determined
+    /// before it is sent. Requests/responses sent in streaming mode, where headers are sent immediately
+    /// but the content of the body is streamed later, will receive a `Transfer-Encoding: chunked`
+    /// to accommodate the dynamic generation of the body.
+    Automatic = 0,
+
+    /// Use the exact framing headers set in the message, falling back to [`Automatic`][`Self::Automatic`]
+    /// if invalid.
+    ///
+    /// In "from headers" mode, any `Content-Length` or `Transfer-Encoding` headers will be honored.
+    /// You must ensure that those headers have correct values permitted by the
+    /// [HTTP/1.1 specification][spec]. If the provided headers are not permitted by the spec,
+    /// the headers will revert to automatic mode and a log diagnostic will be issued about what was
+    /// wrong. If a `Content-Length` is permitted by the spec, but the value doesn't match the size of
+    /// the actual body, the body will either be truncated (if it is too long), or the connection will
+    /// be hung up early (if it is too short).
+    ///
+    /// [spec]: https://datatracker.ietf.org/doc/html/rfc7230#section-3.3.1
+    ManuallyFromHeaders = 1,
+}
+
+impl Default for FramingHeadersMode {
+    fn default() -> Self {
+        Self::Automatic
+    }
+}
+
 /// Optional override for response caching behavior.
 #[derive(Clone, Debug)]
 pub enum CacheOverride {
